=begin
(c) TIG 2011
Permission to use, copy, modify, and distribute this software for 
any purpose and without fee is hereby granted, provided the above
copyright notice appear in all copies.

THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
___________________________________________________________________________

  OnScreenRollingReel.rb
  
Place it in the ../Plugins/ folder and restart Sketchup.

It places a piece of screen-text in the top left corner of the screen.
It lists the last 5 Tools used and their short-cut keys [if any]
e.g.
Select 'Space'
Line 'L'
Move 'M'

[You can alter the length of the list by changing the '5' in Line#52 below]
The most recent Tool is the last in the list.
Plugins etc get listed as '3rd Party Ruby...'
If you erase the screen-text it reappears when you next use
another Tool.
If is 'off' at the startup - you can toggle it on/off with the Plugins
menu item 'On Screen Rolling Reel Toggle'...
___________________________________________________________________________

Donations:
by PayPal to info @ revitrev.org
___________________________________________________________________________

Version:
1.0 20110419 First Release.
1.1 20110420 On/Off Plugins menu toggle added
___________________________________________________________________________
=end
require 'sketchup.rb'
module ToolsText
 class ToolsObserver < Sketchup::ToolsObserver
  def onActiveToolChanged(tools, tool_name, tool_id)
    toolname = self.fix_mac_tool_name(tool_name) 
    return nil if toolname=~/Camera/
    if $toolsText_ToolsObserver_Text_Array[-1]
      return nil if toolname==$toolsText_ToolsObserver_Text_Array[-1]
      return nil if toolname==$toolsText_ToolsObserver_Text_Array[-1]+"Tool"
    end# miss out duplicate tools and camera-ops like orbit/pan etc...
    $toolsText_ToolsObserver_Text_Array << toolname
    $toolsText_ToolsObserver_Text_Array.shift if $toolsText_ToolsObserver_Text_Array[5]### show only last 5 tools used
    shortcuts=Sketchup.get_shortcuts
    scuts={}
    shortcuts.each{|e|
      mkeys=e.split("\t")[0]
      mtool=e.split("/")[-1]
      mtool='PushPull' if mtool=='Pull' ### traps '/' in menu title!
      scuts[mtool]=mkeys
    }
    txt=''
    $toolsText_ToolsObserver_Text_Array.each{|e|
      if e=="SketchTool"
        ee="Line"
      elsif e=="EraseTool"
        ee="Eraser"
      elsif e=="SelectionTool"
        ee="Select"
      elsif e=="ExtrudeTool"
        ee="Follow Me"
      elsif e=="PaintTool"
        ee="Paint Bucket"
      elsif e=="PolyTool"
        ee="Polygon"
      elsif e=="MeasureTool"
        ee="Tape Measure"
      elsif e=="DimensionTool"
        ee="Dimensions"
      elsif e=="SketchCSTool"
        ee="Axes"
      elsif e=="ComponentCSTool"
        ee="Component Axes"
      elsif e=="3DTextTool"
        ee="3D Text"
      elsif e=="RubyTool"
        ee="Plugin..." ###???
      else
        ee=e
      end
      ee.gsub!(/Tool$/,'')
      ### swap for menu name
      scut=scuts[ee]
      if not scut
        scut=''
      else
        scut=" '"+scut+"'"
      end
      txt=txt+ee+scut+"\n"
    }
    ToolsText.add_screen_text() if not $toolsText_Screen_Text or not $toolsText_Screen_Text.valid?
    $toolsText_Screen_Text.text=txt
  end
  def fix_mac_tool_name(tool_name)
    if tool_name == "eTool"
     tool_name = "ScaleTool"
    elsif tool_name == "ool"
     tool_name = "MoveTool"
    elsif tool_name == "onentCSTool"
     tool_name = "ComponentCSTool"
    elsif tool_name == "PullTool"
     tool_name = "PushPullTool"
    end
    return tool_name.to_s
  end
 end#class
 def self.add_screen_text()
   $toolsText_Screen_Text=Sketchup.active_model.add_note(" ", 0.01, 0.02)
 end
 def self.menu()
   $toolsText_Screen_Text_Menu=false if not $toolsText_Screen_Text_Menu
   if $toolsText_Screen_Text_Menu==false
     $toolsText_Screen_Text_Menu=true
     $toolsText_ToolsObserver_Text_Array=[]
     self.add_screen_text()
     Sketchup.active_model.tools.remove_observer($toolsText_ToolsObserver)
     $toolsText_ToolsObserver=ToolsText::ToolsObserver.new()
     Sketchup.active_model.tools.add_observer($toolsText_ToolsObserver)
     self.addToolsTextAppObserver()
   else
     $toolsText_Screen_Text_Menu=false
     $toolsText_ToolsObserver_Text_Array=[]
     $toolsText_Screen_Text.erase! if $toolsText_Screen_Text and $toolsText_Screen_Text.valid?
     Sketchup.active_model.tools.remove_observer($toolsText_ToolsObserver)
   end#if
 end
 # Attach the App observer at kick-off
 def self.addToolsTextAppObserver()
   $toolsTextAppObserver=ToolsTextAppObserver.new() if not $toolsTextAppObserver
   Sketchup.remove_observer($toolsTextAppObserver)
   Sketchup.add_observer($toolsTextAppObserver)
 end#def
 ### add menu item
 if not file_loaded?(File.basename(__FILE__))
   menu=UI.menu("Plugins").add_item("On Screen Rolling Reel Toggle"){ToolsText::menu}
 end#if
 file_loaded(File.basename(__FILE__))
 ###
end#module

##########################################################
### AppObserver
class ToolsTextAppObserver < Sketchup::AppObserver
  def onNewModel(model)  ### from menu or app
    $toolsText_Screen_Text.erase! if $toolsText_Screen_Text and $toolsText_Screen_Text.valid?
    ToolsText::addToolsTextAppObserver() if $toolsText_Screen_Text_Menu
  end
  def onOpenModel(model) ### from menu
    $toolsText_Screen_Text.erase! if $toolsText_Screen_Text and $toolsText_Screen_Text.valid?
    ToolsText::addToolsTextAppObserver() if $toolsText_Screen_Text_Menu
  end
  def onQuit()
    $toolsText_Screen_Text.erase! if $toolsText_Screen_Text and $toolsText_Screen_Text.valid?
    Sketchup.active_model.tools.remove_observer($toolsText_ToolsObserver)
  end
end
#########################